%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% MVEnergy.m - Computes the forward energy in the right image that
% corresponds to the 3 cases of seam directions in the left image
%
% Input:  Disp_2L - Two lines of the disparity map 
%         I_2L    - Two lines of the right image
%
% Output: E1, E2, E1 - The vertical forward enery that corresponds to each
%                      the possible direction of seam in the left image
%         HoriznE    - The horiziontal energy in the right image

function [E1,E2,E3,HoriznE] = MVEnergy(Disp_2L, I_2L)

[m,n,view_num] = size(Disp_2L);
j = 1:n ;
X = MapDisp(Disp_2L);

% the energy that correspond to each of the cases (terms 1-3)
E1 = Inf(1,n); 
E2 = Inf(1,n);
E3 = Inf(1,n);

% The source is j: The coordinates in the right image that corresponds to 1:n in the left
X0 = X(2,j);
% the source is j-1: The coordinates in the right image that corresponds
% to it
X1 = [0 X(1,j(2:end)-1)];
% The coordinates in the second image that corresponds to 1:n in the ref
X2 = X(1,j);
% The coordinates in the second image that corresponds to 2:n in the ref
X3 = [X(1,j(1:n-1)+1) 0];

[SLeftDiffVec, SRightDiffVec]  = DiffVector(I_2L);
LeftCumSum = [cumsum(SLeftDiffVec(1:end-1)) Inf];
RightCumSum = [Inf cumsum(SRightDiffVec(2:end))];

Leftidx1 = find(X1 < X0 & X0 < n+1 & X1 > 1); % the source is on the left of the current pixel
Rightidx1 = find(X1 > X0 & X1 <n+1 & X0 > 1); % the source is on the right of the current pixel
EqInd1 = find(X1 == X0 & X0 <n+1 & X0 > 1); 

Leftidx2 = find(X2 < X0 & X0 <n+1 & X2 > 1);
Rightidx2 = find(X2 > X0 & X2 <n+1 & X0 > 1);
EqInd2 = find(X2 == X0 & X0 <n+1 & X0 > 1); 

Leftidx3 = find(X3 < X0 & X0 <n+1 & X3 > 1);
Rightidx3 = find(X3 > X0 & X3 <n+1 & X0 > 1);
EqInd3 = find(X3 == X0 & X0 <n+1 & X0 > 1); 

%ind = [1:n]';
% E1(Leftidx1)  = interp1q(ind,LeftCumSum',X0(Leftidx1)'-1) - interp1q(ind,LeftCumSum',X1(Leftidx1)'-1);
% E1(Rightidx1) = interp1q(ind,RightCumSum',X1(Rightidx1)'-1) - interp1q(ind,RightCumSum',X0(Rightidx1)'-1);
% 
% E2(Leftidx2) = interp1(ind,LeftCumSum',X0(Leftidx2)'-1) - interp1(ind,LeftCumSum',X2(Leftidx2)'-1);
% E2(Rightidx2) = interp1(ind,RightCumSum',X2(Rightidx2)'-1) - interp1(ind,RightCumSum',X0(Rightidx2)'-1);
% 
% E3(Leftidx3) = interp1(ind,LeftCumSum',X0(Leftidx3)'-1) - interp1(ind,LeftCumSum',X3(Leftidx3)'-1);
% E3(Rightidx3) = interp1(ind,RightCumSum',X3(Rightidx3)'-1) - interp1(ind,RightCumSum',X0(Rightidx3)'-1);

E1(Leftidx1)  = LeftCumSum(ceil(X0(Leftidx1)'-1)) - LeftCumSum(ceil(X1(Leftidx1)'-1));
E1(Rightidx1) = RightCumSum(ceil(X1(Rightidx1)')) - RightCumSum(ceil(X0(Rightidx1)'));
if(~isempty(EqInd1))
    E1(EqInd1) = 0;
end
E2(Leftidx2) = LeftCumSum(ceil(X0(Leftidx2)'-1)) - LeftCumSum(ceil(X2(Leftidx2)'-1));
E2(Rightidx2) = RightCumSum(ceil(X2(Rightidx2)')) - RightCumSum(ceil(X0(Rightidx2)'));
if(~isempty(EqInd2))
    E2(EqInd2) = 0;
end

E3(Leftidx3) = LeftCumSum(ceil(X0(Leftidx3)'-1)) - LeftCumSum(ceil(X3(Leftidx3)'-1));
E3(Rightidx3) = RightCumSum(ceil(X3(Rightidx3)')) - RightCumSum(ceil(X0(Rightidx3)'));
if(~isempty(EqInd3))
    E3(EqInd3) = 0;
end

%  HoriznE = abs(interp1([1:n],I_2L(2,:),X0-1) - interp1([1:n],I_2L(2,:),X0+1));
rX0 = round(X0);
ind = find(rX0 > 1 & rX0 < n); 
HoriznE(ind) = abs(I_2L(2,rX0(ind)-1) - I_2L(2,rX0(ind)+1));
E1(isnan(E1)) = Inf;
E2(isnan(E2)) = Inf;
E3(isnan(E3)) = Inf;
HoriznE(isnan(HoriznE)) = Inf;
end