%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% findOptimalSeam.m - Finds optimal seam, given a dynamic programming
% matrix, and the source map (to deal with genrilazed seams)
%
% Output: x - vector of optimal seam in the left image
%
% Matrix M contains the output of the Dynamic Programming function.
% The vector x contains the pixels of the optimal (=low cost) seam.
% The pixels of the seam can be computed as (i,x(i))
% The seam computed is a vertical seam.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function x = findOptimalSeam(M, SourceMap)

m = size(M,1);
n = size(M,2);
x = zeros(m,1);

% Find minimum value in last row
[dummy,x(m)] = min(M(m,:));
if(dummy > 1e9)
    disp('Reached the limit! can remover seam')
    x = -1
    return;
end
% Back track
for i=m-1:-1:1,
 
    if(SourceMap(i+1,x(i+1)) )
        x(i) = SourceMap(i+1,x(i+1));
    else
        a = max(x(i+1)-1,1);
        b = min(x(i+1)+1,n);
        [dummy,offset] = min(M(i,a:b));
        x(i) = a-1+offset;
    end
end

return
