%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% runme.m - demo for stereo seam carving code
% Input: test.txt  - a batch file containing the input commands. 
%                    Each line in test.txt is a running command with
%                    possibly different input parameters
%                    See test\test.txt for example and syntax description 
% Author: Tali Basha (talibasha@gmail.com)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

addpath(genpath(cd),1);
save_results = 0;
disp_scale = -1./3;
% the input batch file and folder paths
TestFolder = 'test\';
TxtFile = 'test.txt'; 

TxtPath = [TestFolder '\' TxtFile];
F = read_input(TxtPath);
TestNum = length(F);
display = 1;

for i=1:TestNum  
    f = [F{i}.ImageFolder '\' F{i}.IL];
    IL = im2double(imread(f));
    f = [F{i}.ImageFolder '\' F{i}.IR];
    IR = im2double(imread(f));
  
    alpha = F{i}.alpha;
    beta = F{i}.beta;
    gamma = F{i}.gamma;
    pn = F{i}.pn;
    
   [m,n,c] = size(IL);
    
    Disp_in = zeros(m,n);
    if(isempty(F{i}.Disp))
        Disp_in = SGM(IL, IR);
    else
        f = [F{i}.ImageFolder '\' F{i}.Disp];
        Disp = double(imread(f));
        Disp_in = disp_scale*Disp;     
    end
    
    % preprocessing the depth map
    se = strel('disk',2);
    Disp_in = imopen(Disp_in, se);
    Disp_in = FillDisp(Disp_in);
    
    
    dn = round(pn*n);
    w = n-dn;
    [SL,SR,Disp_o] = StereoSeamCarving(IL,IR,Disp_in,w, alpha, beta, gamma, display);
    
    IL_o = imretarget(IL,SL,dn);
    IR_o = imretarget(IR,SR,dn);    
    % saves the results in the same folder as the test.txt file is
    save_res;
end
